@@ -6,6 +6,7 @@
 
 // Forward QT includes
 class QString;
+class QSocketNotifier;
 
 #include "mixer_backend.h"
 
@@ -14,7 +15,7 @@ class Mixer_ALSA : public Mixer_Backend
 	public:
 		Mixer_ALSA( int device = -1 );
 		~Mixer_ALSA();
-		
+
 		virtual int  readVolumeFromHW( int devnum, Volume &vol );
 		virtual int  writeVolumeToHW( int devnum, Volume &vol );
 		virtual bool setRecsrcHW( int devnum, bool on);
@@ -22,12 +23,13 @@ class Mixer_ALSA : public Mixer_Backend
 	        virtual void setEnumIdHW(int mixerIdx, unsigned int);
       		virtual unsigned int enumIdHW(int mixerIdx);
 		virtual bool prepareUpdateFromHW();
+                virtual bool needsPolling() { return false; }
+                virtual void prepareSignalling( Mixer *mixer );
 
-		
 	protected:
 		virtual int open();
 		virtual int close();
-		
+
 	private:
 		int identify( snd_mixer_selem_id_t *sid );
 		snd_mixer_elem_t* getMixerElem(int devnum);
@@ -41,6 +43,10 @@ class Mixer_ALSA : public Mixer_Backend
                 bool _initialUpdate;
 		snd_mixer_t *_handle;
 		QString devName;
+	        struct pollfd  *m_fds;
+                QSocketNotifier **m_sns;
+		int m_count;
+
 };
 
 #endif
