@@ -27,6 +27,7 @@
 #include <qlayout.h>
 #include <qframe.h>
 #include <qpushbutton.h>
+#include <qdatetime.h>
 
 // KDE
 #include <kdebug.h>
@@ -43,7 +44,7 @@
 //    Users will not be able to close the Popup without opening the KMix main window then.
 //    See Bug #93443, #96332 and #96404 for further details. -- esken
 ViewDockAreaPopup::ViewDockAreaPopup(QWidget* parent, const char* name, Mixer* mixer, ViewBase::ViewFlags vflags, KMixDockWidget *dockW )
-      : ViewBase(parent, name, mixer, WStyle_Customize | WType_Popup | Qt::WStyle_DialogBorder, vflags), _mdw(0), _dock(dockW)
+      : ViewBase(parent, name, QString::null, mixer, WStyle_Customize | WType_Popup | Qt::WStyle_DialogBorder, vflags), _mdw(0), _dock(dockW)
 {
     QBoxLayout *layout = new QHBoxLayout( this );
     _frame = new QFrame( this );
@@ -53,6 +54,7 @@ ViewDockAreaPopup::ViewDockAreaPopup(QWidget* parent, const char* name, Mixer* m
     _frame->setLineWidth( 1 );
 
     _layoutMDW = new QGridLayout( _frame, 1, 1, 2, 1, "KmixPopupLayout" );
+    _hideTimer = new QTime();
     init();
 }
 
@@ -61,7 +63,7 @@ ViewDockAreaPopup::~ViewDockAreaPopup() {
 
 
 
-void ViewDockAreaPopup::mousePressEvent(QMouseEvent *)
+void ViewDockAreaPopup::mousePressEvent(QMouseEvent *me)
 {
 //	kdDebug() << "Teste pres mouse" << endl;
     /**
@@ -72,11 +74,17 @@ void ViewDockAreaPopup::mousePressEvent(QMouseEvent *)
        properly take care of it in QWidget.
     */
     if ( ! this->hasMouse() ) {
+        _hideTimer->start();
         hide(); // needed!
     }
     return;
 }
 
+bool ViewDockAreaPopup::justHidden()
+{
+    return _hideTimer->elapsed() < 300;
+}
+
 void ViewDockAreaPopup::wheelEvent ( QWheelEvent * e ) {
    // Pass wheel event from "border widget" to child
    if ( _mdw != 0 ) {
@@ -102,7 +110,7 @@ void ViewDockAreaPopup::setMixSet(MixSet *)
     //    kdDebug(67100) << "ViewDockAreaPopup::setMixSet()\n";
     // This implementation of setMixSet() is a bit "exotic". But I will leave it like this, until I implement
     // a configuration option for "what device to show on the dock area"
-    _dockDevice = (*_mixer)[_mixer->masterDevice()];
+    _dockDevice = _mixer->masterDevice();
     if ( _dockDevice == 0 ) {
         // If we have no mixer device, we will take the first available mixer device
         _dockDevice = (*_mixer)[0];
@@ -190,15 +198,7 @@ void ViewDockAreaPopup::refreshVolumeLevels() {
 }
 
 void ViewDockAreaPopup::showPanelSlot() {
-	if( ! _dock->parentWidget()->isVisible() )
-	{
-		_dock->parentWidget()->show();
-	}
-	else
-	{
-		_dock->parentWidget()->hide();
-	}
-		
+	_dock->toggleActive();
 	_dock->_dockAreaPopup->hide();
 }
 
