@@ -12,7 +12,12 @@
 
 
 #include "inputStream.h"
-#include "../util/mmx.h"
+#include "../util/mmx/mmx.h"
+
+#include <iostream>
+
+using namespace std;
+
 
 InputStream::InputStream() {
   timeStampArray=new TimeStampArray((char*)"Input",3000);
