@@ -41,7 +41,7 @@ MpegAudioInfo::MpegAudioInfo(FileAccess* input) {
 
 
 MpegAudioInfo::~MpegAudioInfo() {
-  delete (xHeadData->toc);
+  delete[] (xHeadData->toc);
   delete xHeadData;
   delete id3;
   delete mpegAudioStream;
