@@ -14,7 +14,7 @@
 
 extern "C"
 {
-	Plugin *create_plugin()
+	KDE_EXPORT Plugin *create_plugin()
 	{
 		return new InfraRed();
 	}
@@ -98,11 +98,21 @@ void InfraRed::slotCommand(const QString &remote, const QString &button, int rep
 
 	case IRPrefs::SeekForward:  // + 3 seconds
 		napp->player()->skipTo( QMIN(napp->player()->getLength(),
-									 napp->player()->getTime() + 3000) );
+		                             napp->player()->getTime() + 3000) );
+		break;
+	case IRPrefs::ShowPlaylist:
+		napp->player()->toggleListView();
+		break;
+	case IRPrefs::NextSection:
+		// This and the next case theoretically shouldn't bypass player()
+		// but I'm making this change as inobtrusive as possible. That
+		// means restricting the change to infrared and not messing around
+		// in libnoatun -- Neil
+		napp->playlist()->nextSection();
+		break;
+	case IRPrefs::PreviousSection:
+		napp->playlist()->previousSection();
 		break;
-    case IRPrefs::ShowPlaylist:
-        napp->player()->toggleListView();
-        break;
 	}
 }
 
